﻿:namespace SaltInSpice
⍝ Implement SALT commands within Spice
    ⎕ml ⎕io←1
    Cnames←'Compare' 'Open' 'RemoveVersions' 'List' 'Load' 'Save' 'Settings' 'Snap'
    ∇ r←List
      :Access Shared Public
      r←⎕NS¨(⍴Cnames)⍴⊂''
      r.(Group Parse)←⊂'SALT' ''
      r.Name←Cnames
      r[1].Desc←'Compare versions of 2 files'
      r[2].Desc←'Open a file'
      r[2].Parse←'1s -with=' ⍝ 'open' takes max 1 argument and a possible switch with which accepts a value
      r[3].Desc←'Remove some or all versions'
      r[4].Desc←'List objects or folders'
      r[5].Desc←'Load an object (class or namespace or fn)'
      r[6].Desc←'Save an object (class or namespace or fn)'
      r[7].Desc←'Change local and registry settings'
      r[8].Desc←'Save changes made to the workspace'
    ∇

    ∇ Û←Run(Ûcmd Ûargs);⎕USING;skipifempty;file;Path;z;with
      :Access Shared Public
      :If Ûcmd≡'Open'
          ⎕USING←'System.Diagnostics,system.dll'
          skipifempty←1
          :If skipifempty<0∊⍴file←1⊃Ûargs.Arguments,⊂''
              Path←(2 ⎕NQ'.' 'GetEnvironment' 'homepath'),'\Documents\' ⍝ default dir
              'F'⎕WC'FileBox' 'Open file'Path('Event' 'FileBoxOK' 1)
              :If 3>⍴z←⎕DQ'F'
              :OrIf 'FileBoxOK'≢2⊃z
                  →0
              :EndIf
              file←3⊃z
          :EndIf
          with←Ûargs.with
          :If ⎕SE.SALTUtils.isRelPath file
              Û←⎕SE.SALT.Explore file,(0≢with)/' -use=',with
          :Else
              Û←{6::'' ⋄ 90::⎕EXCEPTION.Message ⋄ Process.Start(0≡with)↓with file}⍬
          :EndIf
      :Else
        ⍝ Simulate calling directly from the original ns
          ⎕CS ##.THIS ⍝ we know THIS has been set for us
          Û←⍎'⎕SE.SALT.',Ûcmd,' Ûargs' ⍝ Û vars for Snap
      :EndIf
    ∇

    ∇ r←level Help Cmd
      :Access Shared Public
      (1↑Cmd)←⎕SE.Dyalog.Utils.ucase 1↑Cmd
      :If 'Open'≡Cmd
          r←'Open a file using the default program, or a specified program.'
          r←↑r'Relative pathnames are deemed to be SALT files'
      :Else
          r←⍎'⎕SE.SALT.',Cmd,' ''?'''
      :EndIf
      →level↓0
      r←,r,CR←⎕AV[4]
      r,←,CR,'Examples',[0.1]'-'
      :Select Cmd
      :Case 'Save'
          r,←CR,'Save "myobj" in <myfile> (not using a version number):'
          r,←CR,'      ]save myobj /my/folder/myfile'
          r,←CR,'Start using version numbers with "myobj":'
          r,←CR,'      ]save myobj -version'
          r,←CR,'Save "myobj" at a specific version number (here 12):'
          r,←CR,'      ]save myobj -ver=12'
          r,←CR,'Save XYZ to folder ensuring folder is created is non existant:'
          r,←CR,'      ]save XYZ /new/folder/xyz -makedir'
          r,←CR,'Save namespace NN and convert it to script form in the workspace:'
          r,←CR,'      ]save NN  relative\path\nn  -convert'
      :Case 'RemoveVersions'
          r,←CR,'Remove version 12 of "myfile":'
          r,←CR,'      ]removeversions /my/folder/myfile -version=12'
          r,←CR,'Remove versions above 12 of "myfile":'
          r,←CR,'      ]removeversions /my/folder/myfile -version=>12'
          r,←CR,'Remove versions 12 to 21 of "myfile":'
          r,←CR,'      ]removeversions /my/folder/myfile -version=12-21'
          r,←CR,'Remove versions starting at 12 of "myfile" except the last which will be renamed version 12:'
          r,←CR,'      ]removeversions /my/folder/myfile -version=≥12 -collapse'
          r,←CR,'Remove all versions of "myfile" except the last one which will be renamed the lowest actual version:'
          r,←CR,'      ]removeversions /my/folder/myfile -all'
      :Case 'Snap'
          r,←CR,'Save every new item in the ws to path specified; changes items are saved to their own location:'
          r,←CR,'      ]snap C:\new\project'
          r,←CR,'Same thing but make sure the folder exists and do not prompt for confirmation of replacement:'
          r,←CR,'      ]snap C:\new\project -makedir -noprompt'
          r,←CR,'Save only class 9 objects that start with "GUI" and convert namespaces to script form in the ws:'
          r,←CR,'      ]snap /utils/gui -class=9 -pattern=GUI* -convert'
          r,←CR,'Save every new object in a file whose prefix is "UTIL" (eg fn <cat> goes to <UTILcat>):'
          r,←CR,'      ]snap /basics -fileprefix=UTIL'
          r,←CR,'Save every item, including global variables which are stored in a separate file called <globals>'
          r,←CR,'and create a load fn named <lx> to recreate the ws:'
          r,←CR,'      ]snap /my/wsx -vars=globals -loadfn=lx'
      :Case 'Load'
          r,←CR,'Load and define the object in a perticular file:'
          r,←CR,'      ]load \tmp\myobject'
          r,←CR,'Load all version 3 of files starting with TM:'
          r,←CR,'      ]load TM* -ver=3'
          r,←CR,'Bring script S, don''t define it, don''t keep a SALT trace and assign its reference to X:'
          r,←CR,'      ]X←load \tmp\S -noname -nolink'
          r,←CR,'Disperse the objects in /tmp/ns in #.N:'
          r,←CR,'      ]load /tmp/ns -disperse -target=#.N'
      :Case 'Compare'
          r,←CR,'Compare class in file C with object in workspace:'
          r,←CR,'      ]compare /my/files/C -version=ws'
          r,←CR,'Compare versions 2 and 7 of file X; use +- as line symbols:'
          r,←CR,'      ]compare \tmp\X -ver=2 7 -symb=+-'
          r,←CR,'Compare file A with file B, use always external pgm BC to compare:'
          r,←CR,'      ]comp /tmp/A  /myfiles/B -use=/bin/BC -permanent'
      :Case 'Open'
          r,←CR,'Open disk explorer to show the contents of /tmp:'
          r,←CR,'      ]open  /tmp'
          r,←CR,'Open file \my\file with Notepad (Windows):'
          r,←CR,'      ]open  \my\file'
          r,←CR,'Open file X with program <vi>:'
          r,←CR,'      ]open X -with=vi'
      :Case 'List'
          r,←CR,'List all files in \tmp and all files below:'
          r,←CR,'      ]list \tmp -recursive'
          r,←CR,'List folders of folder Y:'
          r,←CR,'      ]list \myfiles\Y -folders'
          r,←CR,'List versions and type of files in relative folder Z:'
          r,←CR,'      ]list  Z  -type  -ver'
          r,←CR,'List the full (starting from #)  path of files:'
          r,←CR,'      ]list  -full=2'
      :Case 'Settings'
          r,←CR,'Find the current settings for "workdir":'
          r,←CR,'      ]settings workdir'
          r,←CR,'Overwrite the paths for User Commands:'
          r,←CR,'      ]sett  cmddir  \my\ucmd1;\my\ucmd2'
          r,←CR,'Set permanently the comparison program (for cmd Compare) to BestC:'
          r,←CR,'      ]set comp \programs\Bestc.exe  -permanent'
          r,←CR,'Remove the requirement to answer prompts when saving objects:'
          r,←CR,'      ]set edprompt 0'
     
      :EndSelect
    ∇

:Endnamespace