﻿:Class wsutils

    ⎕IO←1 ⋄ ⎕ML←0 ⍝  Set those here to avoid inheriting them from outside

    ∇ r←List ⍝ this function usually returns 1 or more namespaces (here only 1)
      :Access Shared Public
      r←⎕NS¨3⍴⊂''                           ⍝ create 2 commands
      r.Name←'defs' 'open' 'dmx'            ⍝ their names
      r[1].Desc←'List single line Dfns'     ⍝ descriptions
      r[2].Desc←'Open a file'
      r[3].Desc←'Display information about the last error'
      r.Group←'wsutils' 'Tools' 'Tools'
     ⍝ Parsing rules for each:
      r[1].Parse←'99s'       ⍝ 'defs' accepts args to limit output
      r[2].Parse←'1s -with=' ⍝ 'open' takes max 1 argument and a possible switch with which accepts a value
      r[3].Parse←'1S -full'  ⍝ 'explain' takes max 1 argument and a possible switch with which does NOT accept a value
    ∇

    ∇ r←Run(Cmd Args);⎕USING;with;file;cr;t;full;arg;dmx;ose;F;dir;Path;z;skipifempty;b;n
      :Access Shared Public
      r←0 0⍴''
      :Select Cmd
      :Case 'dmx'
          r←'⎕DMX only available starting in V13.1'
          →0⌿⍨0≡dmx←##.DMX
          arg←1⊃Args.Arguments,⊂''
          :If dmx.ENX=0 ⋄ r←'No further help available' ⋄ →0 ⋄ :EndIf
          r←dmx.DM
          :If Args.full
              r,←⊂dmx.('[',Category,' error #',(,'ZI4'⎕FMT ENX),']',((Vendor≢'Dyalog')/'[Vendor=',Vendor,']'),': ',Message)
              :If (0 0≢2↑ose←dmx.OSError)
                  t←(1+1⊃ose)⊃'C-library errno()' 'Windows GetLastError()'
                  r,←⊂'[',t,' returned ',(⍕2⊃ose),']: ',3⊃ose
              :EndIf
              r,←⊂'      Internal location: ',⍕dmx.InternalLocation
              r,←⊂'      APL Version:',(⍕'.'⎕WG'APLVersion'),' (BuildID=',(2 ⎕NQ'.' 'GetBuildID'),')'
              r,←⊂'More help at: ',dmx.HelpURL
              r←⍪r
     
          :Else ⍝ Not full
              r←dmx.Message
              :If (0 0≢2↑ose←dmx.OSError)
                  r←r,': ',3⊃ose
              :EndIf
          :EndIf
     
      :Case 'defs'
          r←n{⍺,'←',⍵}¨##.THIS ⎕SE.Dyalog.Utils.drvSrc¨n←##.THIS.⎕NL-3.3
          cr←##.THIS.⎕CR¨t←##.THIS.⎕NL-3.2 4.2 ⋄ n,←t
          t←r,t{1≠⊃⍴⍵:⍺,' (multi-line)' ⋄ ,⍵}¨cr
          t←t[⍋↑n]
          r←'(none found)'
          →0↓⍨∨/b←(0∊⍴Args.Arguments)∨∨⌿1∊¨Args.Arguments∘.⍷⍕¨t
          r←⍪{(+/∧\' '=⍵)↓⍵}¨b⌿t
     
      :Case 'open'
          ⎕USING←'System.Diagnostics,system.dll'
          skipifempty←1
          :If skipifempty<0∊⍴file←1⊃Args.Arguments,⊂''
              Path←(2 ⎕NQ'.' 'GetEnvironment' 'homepath'),'\Documents\' ⍝ default dir
              'F'⎕WC'FileBox' 'Open file'Path('Event' 'FileBoxOK' 1)
              :If 3>⍴z←⎕DQ'F'
              :OrIf 'FileBoxOK'≢2⊃z
                  →0
              :EndIf
              file←3⊃z
          :EndIf
          with←Args.with
          :If ⎕SE.SALTUtils.isRelPath file
              ⎕SE.SALT.Explore file,(0≢with)/' -use=',with
          :Else
              r←{6::'' ⋄ r←Process.Start(0≡with)↓with file ⋄ r}⍬
          :EndIf
      :EndSelect
    ∇

    ∇ r←Help Cmd
      :Access Shared Public
      :Select Cmd
      :Case 'defs'
          r←↑'List definitions of all dfns and d-ops' 'The argument only includes the string given'
      :Case 'open'
          r←'Open a file using the default program, or a specified program.'
          r←↑r'Relative pathnames are deemed to be SALT files'
      :Case 'dmx'
          r←'Provide information about the most recent APL error.'
      :EndSelect
    ∇

:EndClass